--
-- INSERT
--
CREATE OR REPLACE FUNCTION orders_insert() RETURNS TRIGGER
SECURITY DEFINER LANGUAGE PLPGSQL AS '
BEGIN
  INSERT INTO mv(o_orderkey, o_custkey, o_orderstatus, o_totalprice,
    o_orderdate, o_orderpriority, o_clerk, o_shippriority, o_comment)
  VALUES(NEW.o_orderkey, NEW.o_custkey, NEW.o_orderstatus, NEW.o_totalprice,
    NEW.o_orderdate, NEW.o_orderpriority, NEW.o_clerk, NEW.o_shippriority, NEW.o_comment);
  RETURN NULL;
END;
';

DROP TRIGGER orders_insert ON orders;
CREATE TRIGGER orders_insert AFTER INSERT ON orders
FOR EACH ROW EXECUTE PROCEDURE orders_insert();

--
-- UPDATE
--
CREATE OR REPLACE FUNCTION orders_update() RETURNS TRIGGER
SECURITY DEFINER LANGUAGE PLPGSQL AS '
BEGIN
  UPDATE mv SET
    o_orderkey = NEW.o_orderkey,
    o_custkey = NEW.o_custkey,
    o_orderstatus = NEW.o_orderstatus,
    o_totalprice = NEW.o_totalprice,
    o_orderdate = NEW.o_orderdate,
    o_orderpriority = NEW.o_orderpriority,
    o_clerk = NEW.o_clerk,
    o_shippriority = NEW.o_shippriority,
    o_comment = NEW.o_comment
  WHERE o_orderkey = OLD.o_orderkey;
  RETURN NULL;
END;
';

DROP TRIGGER orders_update ON orders;
CREATE TRIGGER orders_update AFTER INSERT ON orders
FOR EACH ROW EXECUTE PROCEDURE orders_update();

--
-- DELETE
--
CREATE OR REPLACE FUNCTION orders_delete() RETURNS TRIGGER
SECURITY DEFINER LANGUAGE PLPGSQL AS '
BEGIN
  DELETE FROM mv WHERE o_orderkey = OLD.o_orderkey;
  RETURN NULL;
END;
';

DROP TRIGGER orders_delete ON orders;
CREATE TRIGGER orders_delete AFTER DELETE ON orders
FOR EACH ROW EXECUTE PROCEDURE orders_delete();