--
-- INSERT
--
CREATE OR REPLACE FUNCTION lineitem_insert() RETURNS TRIGGER
SECURITY DEFINER LANGUAGE PLPGSQL AS '
DECLARE
  cnt INTEGER;
BEGIN
  IF NEW.l_commitdate < NEW.l_receiptdate THEN
    SELECT INTO cnt count(*) FROM mv WHERE NEW.l_orderkey = o_orderkey;
    IF cnt = 0 THEN
      INSERT INTO mv SELECT * FROM orders WHERE o_orderkey = NEW.l_orderkey;
    END IF;
  END IF;
  RETURN NULL;
END;
';

DROP TRIGGER lineitem_insert ON lineitem;
CREATE TRIGGER lineitem_insert AFTER INSERT ON lineitem
FOR EACH ROW EXECUTE PROCEDURE lineitem_insert();

--
-- UPDATE
--
CREATE OR REPLACE FUNCTION lineitem_update() RETURNS TRIGGER
SECURITY DEFINER LANGUAGE PLPGSQL AS '
DECLARE
  cnt INTEGER;
BEGIN
  SELECT INTO cnt count(*) FROM lineitem WHERE NEW.l_orderkey = l_orderkey AND
    l_commitdate < l_receiptdate;
  IF cnt = 0 THEN
    DELETE FROM mv WHERE o_orderkey = NEW.l_orderkey;
  ELSE
    SELECT INTO cnt count(*) FROM mv WHERE NEW.l_orderkey = o_orderkey;
    IF cnt = 0 THEN
      INSERT INTO mv SELECT * FROM orders WHERE o_orderkey = NEW.l_orderkey;
    END IF;
  END IF;
  RETURN NULL;
END;
';

DROP TRIGGER lineitem_update ON lineitem;
CREATE TRIGGER lineitem_update AFTER UPDATE ON lineitem
FOR EACH ROW EXECUTE PROCEDURE lineitem_update();

--
-- DELETE
--
CREATE OR REPLACE FUNCTION lineitem_delete() RETURNS TRIGGER
SECURITY DEFINER LANGUAGE PLPGSQL AS '
DECLARE
  cnt INTEGER;
BEGIN
  SELECT INTO cnt count(*) FROM mv WHERE OLD.l_orderkey = o_orderkey;
  IF cnt <> 0 THEN
    SELECT INTO cnt count(*) FROM lineitem WHERE OLD.l_orderkey = l_orderkey AND
      l_commitdate < l_receiptdate;
    IF cnt = 0 THEN
      DELETE FROM mv WHERE o_orderkey = OLD.l_orderkey;
    END IF;
  END IF;
  RETURN NULL;
END;
';

DROP TRIGGER lineitem_delete ON lineitem;
CREATE TRIGGER lineitem_delete AFTER DELETE ON lineitem
FOR EACH ROW EXECUTE PROCEDURE lineitem_delete();